
#include <iostream>
using namespace std;
#include "irc_socket.h"
using namespace AlgorithMan;


#define bufsize 255

void DebugMessage(IRC* sender, const string& message)
{
    cout << "\x1b[32m" << message << "\x1b[0m\n";
    cout.flush();
}

void just_print(IRC *sender, string& channel, string& user, string& message)
{
    cout << user << "\t" << message << "\n";
    cout.flush();
}

void someone_joins(IRC *sender, string& channel, string& user)
{
    cout << user << " has joined " << channel << "\n";
    cout.flush();
}

void someone_leaves(IRC *sender, string& channel, string& user, string& quit_message)
{
    cout << user << " has left " << channel << " (" << quit_message << ")\n";
    cout.flush();
}

int main(int argc, char** argv)
{
    if(argc < 3)
    {
        cerr << "usage: " << argv[0] << " user[:pass]@server:port channel";
        return 1;
    }
    
    try
    {
        string username;
        string pass;
        string server;
        string channel;

        int i=0;
        for(;argv[1][i] != '@' && argv[1][i] != '\0' && argv[1][i]; i++)
            username += string(1, argv[1][i]);
        if(argv[1][i] == ':')
            for(i++;argv[1][i] != '@' && argv[1][i] != '\0'; i++)
                pass += string(1, argv[1][i]);
        if(argv[1][i] != '@')
        {
            cerr << argv[0] << ": no username given\nusage: " << argv[0] << " user[:pass]@server:port channel";
            return 1;
        }
        server = string(argv[1]).substr(i+1);
        channel = argv[2];
        
        IRC s(server, username, pass);
        
        s.OnMessage = just_print; // set event handlers
        s.OnSomeoneJoins = someone_joins;
        s.OnSomeoneLeaves = someone_leaves;
        s.OnDebugMessage = DebugMessage;
        
        s.Join(channel);

        char message[bufsize+1];
        while(cin.peek() != -1)
        {
            cin.getline(&message[0], bufsize);
            s.SendMessage(channel, string(message));
        }
        s.Quit();
        return 0;
    }
    catch(char const* msg)
    {
        cerr << msg << endl;
        return 1;
    }
}

