
#include "irc_socket.h"
using namespace std;

namespace AlgorithMan
{

    IRC::IRC(string& server, string& nickname, const string& password)
    {
        OnPingPong = NULL;
        OnMessage = NULL;
        OnPrivateMessage = NULL;
        OnKicked = NULL;
        OnTopicChanged = NULL;
        OnSomeoneJoins = NULL;
        OnSomeoneLeaves = NULL;
        OnSomeoneChanesNicknames = NULL;
        OnDebugMessage = NULL;

        Connect(server);
        Activate();
        if(password != "")
            Send(string("PASS ")+password+"\r\n");
        Send(string("NICK ")+nickname+"\r\n");
        Send(string("USER ")+nickname+" 0 * :"+nickname+"\r\n");
    }

    IRC::IRC(string& server, int port, string& nickname, const string& password)
    {
        OnPingPong = NULL;
        OnMessage = NULL;
        OnPrivateMessage = NULL;
        OnKicked = NULL;
        OnTopicChanged = NULL;
        OnSomeoneJoins = NULL;
        OnSomeoneLeaves = NULL;
        OnSomeoneChanesNicknames = NULL;
        OnDebugMessage = NULL;

        Connect(server, port);
        Activate();
        if(password != "")
            Send(string("PASS ")+password+"\r\n");
        Send(string("NICK ")+nickname+"\r\n");
        Send(string("USER ")+nickname+" 0 * :"+nickname+"\r\n");
    }

    IRC::~IRC()
    {
        Quit();
    }

    void IRC::InternalOnRecieve(char* message, int length)
    {
        static string buffer;
        message[length] = '\0';
        buffer += message;

        for(int i = 0; i+1 < buffer.size(); i++)
        {
            if(buffer[i] == '\r' && buffer[i+1] == '\n')
            {
                InternalOnRecieve_Stage2(buffer.substr(0,i));
                buffer = buffer.substr(i+2,buffer.size()-i-2);
                i=-1;
            }
        }
    }
    
    inline bool whitespace(char x)
    {
        return x == ' ' || x == '\n' || x== '\r' || x=='\t';
    }
    
    void IRC::InternalOnRecieve_Stage2(const string& message)
    {
        if(OnDebugMessage != NULL)
            OnDebugMessage(this, message);
                
        string affected_user;
        string command;
        vector<string> parameters;
        
        // split message up into [:affected_user] command {parameter}[:last_parameter]
        int i=0;
        for(; i < message.size() && whitespace(message[i]); i++);
        if(i < message.size())
        {
            if(message[i] == ':')
            {
                for(i++; i < message.size() && !whitespace(message[i]); i++)
                    affected_user += string(1, message[i]);
                for(int j = 0; j < affected_user.size(); j++)
                {
                    if(affected_user[j] == '!')
                    {
                        affected_user = affected_user.substr(0,j);
                        break;
                    }
                }
            }
        }

        for(; i < message.size() && whitespace(message[i]); i++);
        for(; i < message.size() && !whitespace(message[i]); i++)
            command += string(1, message[i]);
        while(i < message.size())
        {
            string temp;
            for(; i < message.size() && whitespace(message[i]); i++);
            if(i < message.size())
            {
                if(message[i] == ':')
                {
                    parameters.push_back(message.substr(i+1, message.size()-i-1));
                    break;
                }
            }
            for(i++; i < message.size() && !whitespace(message[i]); i++)
                temp += string(1, message[i]);
            if(temp != "")
                parameters.push_back(temp);
        }

        // trigger events
        if(command == "PRIVMSG")
        {
            if(OnMessage != NULL)
                if(parameters.size() > 1)
                    OnMessage(this, parameters[0], affected_user, parameters[1]);
            return;
        }
        
        if(command == "JOIN")
        {
            if(OnSomeoneJoins != NULL)
                if(parameters.size() > 0)
                    OnSomeoneJoins(this, parameters[0], affected_user);
            return;
        }
        
        if(command == "PART")
        {
            if(OnSomeoneLeaves != NULL)
            {
                if(parameters.size() > 1)
                {
                    if(parameters.size() < 2)
                        parameters.push_back("");
                    OnSomeoneLeaves(this, parameters[0], affected_user, parameters[1]);
                }
            }
            return;
        }
        
        if(command == "PING")
        {
            Send("PONG foobar\r\n");
            if(OnPingPong != NULL)
                OnPingPong(this);
            return;
        }

/*
            void (*OnMessage)(IRC* sender, string& channel, string& user, string& text);
            void (*OnPrivateMessage)(IRC* sender, string& user, string& text);
            void (*OnKicked)(IRC* sender, string& channel, string& reason);
            void (*OnTopicChanged)(IRC* sender, string& channel, string& newtopic);
            void (*OnSomeoneJoins)(IRC* sender, string& channel, string& username);
            void (*OnSomeoneLeaves)(IRC* sender, string& channel, string& username, string& quit_message);
            void (*OnSomeoneChanesNicknames)(IRC* sender, string& user, string& newusername);
*/
    }

    // ----------------------------------------------------- methods for clients

    void IRC::Join(string& channel, const string& password)
    {
        if(password != "")
            Send(string("JOIN ")+channel+" "+password+"\r\n");
        else
            Send(string("JOIN ")+channel+"\r\n");
    }
    
    void IRC::Leave(string& channel)
    {
        Send(string("PART ")+channel+"\r\n");
    }

    void IRC::ChangeNickname(string& nickname)
    {
        Send(string("NICK ")+nickname+"\r\n");
    }

    void IRC::SendMessage(string& channel, const string& message)
    {
        Send(string("PRIVMSG ")+channel+" :"+message+"\r\n");
    }
    
    // yes, this is equivalent to SendMessage, but this way the interface is
    // more intuitive (you don't need to know that messages and private
    // messages are technically the same, to be able to send private messages)
    void IRC::SendPrivateMessage(string& recipient, const string& message)
    {
        Send(string("PRIVMSG ")+recipient+" "+message+"\r\n");
    }

    void IRC::SetTopic(string& channel, string& topic)
    {
        Send(string("TOPIC ")+channel+" :"+topic+"\r\n");
    }

    void IRC::Quit()
    {
        Send("QUIT\r\n");
        Disconnect();
    }
    
    // ------------------------------------------- methods for channel operators
    
    void IRC::Kick(string& channel, string& user, const string& reason)
    {
        if(reason == "")
            Send(string("KICK ")+channel+" "+user+"\r\n");
        else
            Send(string("KICK ")+channel+" "+user+" "+reason+"\r\n");
    }

    void IRC::KickBan(string& channel, string& user, const string& reason)
    {
        if(reason == "")
            Send(string("KICK ")+channel+" "+user+"\r\n");
        else
            Send(string("KICK ")+channel+" "+user+" "+reason+"\r\n");
    }

    void IRC::Kill(string& user, const string& reason)
    {
        if(reason == "")
            Send(string("KILL ")+user+"\r\n");
        else
            Send(string("KILL ")+user+" "+reason+"\r\n");
    }

    void IRC::Op(string& channel, string& user)
    {
        Send(string("MODE ")+channel+" "+user+" +o\r\n");
    }

    void IRC::Deop(string& channel, string& user)
    {
        Send(string("MODE ")+channel+" "+user+" -o\r\n");
    }

    void IRC::Voice(string& channel, string& user)
    {
        Send(string("MODE ")+channel+" "+user+" +v\r\n");
    }
    
    void IRC::Mute(string& channel, string& user)
    {
        Send(string("MODE ")+channel+" "+user+" -v\r\n");
    }

    // -------------------------------------------- methods for server operators

    void IRC::BecomeOperator(string& username, string& password)
    {
        Send(string("OPER ")+username+" "+password+"\r\n");
    }

}
