
#ifndef __ALGORITHMAN__IRC_SOCKET_H
    #define __ALGORITHMAN__IRC_SOCKET_H

    #include "socket.h"
    #include<vector>

    namespace AlgorithMan
    {
        class IRC : private Socket
        {
            protected:
                std::string nick;
                void InternalOnRecieve(char* message, int length);
                void InternalOnRecieve_Stage2(const std::string& command);
            public:
                IRC(std::string& server, std::string& nickname, const std::string& password="");
                IRC(std::string& server, int port, std::string& nickname, const std::string& password="");
                ~IRC();

                void Join(std::string& channel, const std::string& password="");
                void ChangeNickname(std::string& newnick);
                void SendMessage(std::string& channel, const std::string& text);
                void SendPrivateMessage(std::string& recipient, const std::string& text);
                void SetTopic(std::string& channel, std::string& topic);
                void Leave(std::string& channel);
                void Quit();

                // methods for channel operators
                void Kick(std::string& channel, std::string& user, const std::string& reason="");
                void KickBan(std::string& channel, std::string& user, const std::string& reason="");
                void Op(std::string& channel, std::string& user);
                void Deop(std::string& channel, std::string& user);
                void Voice(std::string& channel, std::string& user);
                void Mute(std::string& channel, std::string& user);

                // methods for server operator
                void BecomeOperator(std::string& username, std::string& password);
                void Kill(std::string& user, const std::string& reason="");
            
                //vector<std::string> ChannelList();
                //vector<std::string> Users(std::string& channel);

                // remember to set new ones to NULL in constructors
                void (*OnPingPong)(IRC* sender);
                void (*OnDebugMessage)(IRC* sender, const std::string& text);
                void (*OnMessage)(IRC* sender, std::string& channel, std::string& user, std::string& text);
                void (*OnPrivateMessage)(IRC* sender, std::string& user, std::string& text);
                void (*OnKicked)(IRC* sender, std::string& channel, std::string& reason);
                void (*OnTopicChanged)(IRC* sender, std::string& channel, std::string& newtopic);
                void (*OnSomeoneJoins)(IRC* sender, std::string& channel, std::string& username);
                void (*OnSomeoneLeaves)(IRC* sender, std::string& channel, std::string& username, std::string& quit_message);
                void (*OnSomeoneChanesNicknames)(IRC* sender, std::string& user, std::string& newusername);
        };
    }

#endif
