/*
   Socket Class for C++ by Viktor "AlgorithMan" Engelmann
   
   This source code is provided under the terms of the
   GNU general public license version 3. See gpl3.txt for details.

   thanks to
      http://www.zotteljedi.de/
      http://www.linuxhowtos.org/C_C++/socket.htm
*/

#ifndef __ALGORITHMAN__SOCKET_H
    #define __ALGORITHMAN__SOCKET_H

    #include <unistd.h>
    #include <fcntl.h> // open, close, read, write
    #include <stdlib.h> // atoi
    #include <pthread.h> // pthread_t etc.
    #include <arpa/inet.h>
    #include <netdb.h> 
    #include <string>
    #include <string.h> // memcpy

    namespace AlgorithMan
    {
        class Socket
        {
            friend void* MonitorSocket(void* s); // thread-function

            private:
                int socket_handle;
                int port;
                int defaultport;
                bool connected;
                struct sockaddr_in server_address;
                volatile bool active;
                pthread_t monitor;

            protected:
                // overwrite InternalOnRecieve in derived classes to catch
                // protocol-specific messages (like the PING-PONG messages in
                // the IRC Protocol)
                virtual void InternalOnRecieve(char* buffer, int len);

            public:
                Socket();
                Socket(std::string server);
                Socket(std::string& server, int port);
                virtual ~Socket();
                void Connect(std::string& server); // supports "host:port"
                void Connect(std::string& server, int port);
                void Disconnect();
                void Send(const std::string& message);
                std::string Recieve(int max_chars);
                std::string RemoteIP();

                void (*OnRecieve)(Socket* Sender, char* Message, int length); // event handler
                void Activate();
                void Deactivate();
        };
    }

#endif
